#pragma once

#include <map>
#include "SGString.h"

//-----------------------------------------------------------------------------
/**
   @brief セッション管理クラス

   セッションの管理の枠組みを提供します。
   サーバに接続し、ログインを行い、認証が成功した状態の
   サーバとクライアントの関係を「セッション」と呼びます。
   セッションは、実際の接続が切断してしまっても継続して保持されます。
   再接続後、セッションIDを元にセッションを復旧することができます。
   使用されなくなったセッションは一定時間後に破棄されます。
   
   開発者は、必要に応じてこのクラスを継承し、
   セッションのタイムアウト判定の独自拡張をおこなったり、
   セッションごとに情報を追加で保持させることが出来ます。
*/
class PGSession
{
  public:
	PGSession();
	PGSession(const SGString &sessionid, int interval);
	virtual ~PGSession();
	
	void SetSessionId(const SGString& sessionid);
	SGString& GetSessionId();
	
	void Extend();
	
	unsigned int mTimerId;  ///< @brief タイマーID
  private:
	SGString mSessionId;    ///< @brief セッションID
	int mInterval;          ///< @brief タイムアウト時間
};

//-----------------------------------------------------------------------------
/**
@brief イベントハンドラリスト
*/
class PGSessionList
{
  public:
	bool Has( const SGString &sessionid );
	PGSession *Search( const SGString &sessionid );
	
	void Add( const SGString &sessionid, PGSession *session );
	void Remove( const SGString &sessionid );

	void Expire( unsigned int timerid );

  private:
	typedef std::map<SGString,PGSession*> List;               ///< @brief イベントハンドラの可変長配列
	typedef std::map<SGString,PGSession*>::iterator Iterator; ///< @brief @ref List のイテレーター
	List mSessionList; ///< @brief イベントハンドラリスト
};

