#pragma once
#include "SGApplication.h"
#include "PGStationSocket.h"
#include "PGDialog.h"
#include "PGClientSocket.h"
#include "PGSession.h"

// コネクションから認証までのタイムアウト時間(秒)
#define TIMEOUT_CONNECTION	10
// 最後の受信からKEEPALIVE送信までの時間(秒)
#define TIMEOUT_KEEPALIVE   180
// 最後の受信からセッションが削除されるまでの時間(秒)
// ０(ゼロ)は無期限です。※注意：無期限の場合、ステーションの再起動までセッションが無くなりません
#define TIMEOUT_SESSION     3600

/**
@brief ダイアログオブジェクト生成クラス
*/
class PGDialogMaker
{
  public:
	PGDialogMaker();
	virtual ~PGDialogMaker();

	virtual const char* GetFilename( const SGString &dialogname );
	virtual PGDialog* CreateDialog( const SGString &dialogname );
};


/**
@brief PlusG Smart Solution アプリケーション基本クラス

PlusG Smart Solution アプリケーションの基幹となるクラスです。
アプリケーションを開発するには、このクラスを継承した実装クラスを作って下さい。
*/
class PGApplication : public SGApplication
{
  public:
	PGApplication(PGDialogMaker& dm);
	~PGApplication();
	
	// ユーザが継承する関数
	/**
	@brief 初期化時コールバック関数

	アプリケーション初期化後に呼ばれます。
	@return 未定義
	*/
	virtual int OnInit()=0;
	/**
	@brief 認証時コールバック関数

	ユーザー認証時に呼ばれます。
	@param name ユーザー名
	@param pass パスワード
	@param pgsock ソケット
	@return 未定義
	*/
	virtual bool OnAuth( const SGString& name, const SGString& pass, PGClientSocket* pgsock )=0;
	virtual void ApplicationTimer(TimeVal time,unsigned int id);

	virtual bool OnAuthCheck( const SGString& sessionid, PGClientSocket* pgsock );
	virtual void OnStart(PGClientSocket* pgsock,SGString& type ,int width ,int height);
	virtual void OnTimer(TimeVal time,unsigned int id);

	// ユーザが継承しない関数
	unsigned int MakeSocketId();
	unsigned int MakeSessionTimerId();
	int Initialize(int argc, char** argv);
	PGDialog* CreateDialog(const char* dialogname);
	void Authentication(const SGString sessionid);
	bool CreateSession(const SGString& sessionid,int timeout);
	PGDialogList* DialogList();
	PGSessionList* SessionList();
	static char* GetModuleDirectory();

	void SendDataList( const SGString& sessionid, const SGString& dialogname, const SGString& json );
	void RemoveSocketBySession( const SGString& sessionid );
	int GetSocket(unsigned int id,PGClientSocket** socket);
private:
	void DialogTimer(TimeVal time,unsigned int id,SGString name);

  protected:
	PGDialogMaker& mDialogMaker;
	PGStationSocket mStationSocket; ///< @brief 待ち受けソケット
	PGDialogList mDialogList;       ///< @brief ダイアログリスト
	PGSessionList mSessionList;     ///< @brief セッションリスト
	SGStream mLog;                  ///< @brief アプリケーションログ
	unsigned int mSocketId;
	unsigned int mSessionTimerId;
};

extern PGApplication* app;
