#pragma once
//#include "sglib.h"
#include <netinet/ip.h>
//#include <semaphore.h>
//#include <string>
#include "SGLog.h"
#include "SGFdData.h"
#include "SGApplication.h"

using namespace std;

//-----------------------------------------------------------------------------
/**
@brief ソケットの基本クラス
*/
class SGSocket : public SGFdData
{
public:
	SGSocket(SGFdDataList* list);
	SGSocket(int size, SGStream *srteam,SGFdDataList* list);
	virtual ~SGSocket();

	int GetAddrInfo(int flags,int family,int type,const char* addr,int port,struct addrinfo **ai);
	void FreeAddrInfo(struct addrinfo *ai);
	int Socket(int family,int type,int protocol);
	int Bind(const struct sockaddr *addr,socklen_t addrlen);
	int Connect(const struct sockaddr *addr,socklen_t addrlen);
	int Close();

	// 待ち受けの開始
	// 戻り値 0:正常終了, 1:エラー
	int Listen(int backlog=SOMAXCONN);

	// コネクションの取り出し
	// 戻り値 0以上:正常終了, -1以下:エラー
	int Accept();

	int SendTo(const void* data,int len,const struct sockaddr* addr,socklen_t addrlen);
	virtual int Send(const void *data, int length);

	// コールバック(イベント)関数群
	virtual void OnOpen();
	virtual void OnClose();
	virtual void OnListen();
	virtual void OnConnect();
	virtual void OnAccept(int sd, struct sockaddr *addr, socklen_t *addrlen);
	virtual int  OnReceive(struct sockaddr* addr,socklen_t addrlen);
	virtual void OnTimeout(TimeVal time);

	virtual void ReadyToRead();

	SGLog& Log(){ return mLog; }

private:
	int mStatus;
	SGLog mLog;  ///< @brief ソケットログ
};

//-----------------------------------------------------------------------------
/**
@brief UDP ソケットの基本クラス
*/
class SGSocketUdp : public SGSocket
{
public:
	SGSocketUdp(int size, SGStream *stream,SGFdDataList* list);
	virtual ~SGSocketUdp();

	int Open(const char* host,int port,int buffersize=512*1024);
	int SendTo(const void* data, size_t length, const char *host, unsigned short port);

	virtual void OnOpen();
	virtual void OnClose();

private:
	SGApplication* mApp; ///< @brief アプリケーション
};

//-----------------------------------------------------------------------------
/**
@brief サーバー(待ち受け)ソケットの基本クラス
*/
class SGServerSocket : public SGSocket
{
public:
	SGServerSocket(int size, SGStream *stream,SGFdDataList* list);
	virtual ~SGServerSocket();

	int Open(const char* host,int port);
	virtual void OnOpen();
	virtual void OnClose();
	virtual void OnListen();
	virtual void OnAccept(int sd, struct sockaddr *addr, socklen_t *addrlen);

private:
	void ReadyToRead();
};

//-----------------------------------------------------------------------------
/**
@brief クライアント(接続)ソケットの基本クラス
*/
class SGClientSocket : public SGSocket
{
public:
	SGClientSocket(int size, SGStream *stream,SGFdDataList* list);
	virtual ~SGClientSocket();

	int Open(const char* host,int port,int buffersize=512*1024);
	int Create(int sd,int buffersize=512*1024);

	virtual void OnClose();

private:
	void ReadyToWrite();

private:
	SGApplication* mApp; ///< @brief アプリケーション
};
